Imports System.Globalization
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports GalaSoft.MvvmLight
Imports GalaSoft.SimpleSerializer

Namespace Model
    <SimpleSerialize>
    Public Class [Friend]
        Inherits ObservableObject

        ''' <summary>
        ''' The <see cref="FirstName" /> property's name.
        ''' </summary>
        Public Const FirstNamePropertyName As String = "FirstName"

        Private _firstName As String

        ''' <summary>
        ''' Sets and gets the FirstName property.
        ''' Changes to that property's value raise the PropertyChanged event. 
        ''' </summary>
        <SimpleSerialize(FieldName:="first_name")>
        Public Property FirstName As String
            Get
                Return _firstName
            End Get
            Set(ByVal value As String)
                MyBase.Set(FirstNamePropertyName, _firstName, value)
            End Set
        End Property

        ''' <summary>
        ''' The <see cref="LastName" /> property's name.
        ''' </summary>
        Public Const LastNamePropertyName As String = "LastName"

        Private _lastName As String

        ''' <summary>
        ''' Sets and gets the LastName property.
        ''' Changes to that property's value raise the PropertyChanged event. 
        ''' </summary>
        <SimpleSerialize(FieldName:="last_name")>
        Public Property LastName As String
            Get
                Return _lastName
            End Get
            Set(ByVal value As String)
                MyBase.Set(LastNamePropertyName, _lastName, value)
            End Set
        End Property

        ''' <summary>
        ''' The <see cref="DateOfBirth" /> property's name.
        ''' </summary>
        Public Const DateOfBirthPropertyName As String = "DateOfBirth"

        Private _dateOfBirthString As String

        ''' <summary>
        ''' Sets and gets the DateOfBirth property.
        ''' Changes to that property's value raise the PropertyChanged event. 
        ''' </summary>
        <SimpleSerialize(FieldName:="birthday")>
        Public Property DateOfBirthString As String
            Get
                Return _dateOfBirthString
            End Get
            Set(ByVal value As String)
                _dateOfBirthString = value
                RaisePropertyChanged(Function() DateOfBirth)
            End Set
        End Property

        Public Property DateOfBirth As Date
            Get
                If String.IsNullOrEmpty(_dateOfBirthString) Then
                    Return Date.MinValue
                End If

                Return Date.ParseExact(DateOfBirthString, "d", CultureInfo.InvariantCulture)
            End Get
            Set(ByVal value As Date)
                _dateOfBirthString = value.ToString("d", CultureInfo.InvariantCulture)
            End Set
        End Property

        Private _imageUrl As String

        <SimpleSerialize(FieldName:="picture")>
        Public Property ImageUrl As String
            Get
                Return _imageUrl
            End Get
            Set(ByVal value As String)
                _imageUrl = value
                RaisePropertyChanged(Function() ImageUri)
            End Set
        End Property

        Public ReadOnly Property ImageUri As Uri
            Get
                Return New Uri(_imageUrl)
            End Get
        End Property
    End Class
End Namespace
