Imports System.Globalization
Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports GalaSoft.MvvmLight
Imports GalaSoft.MvvmLight.Messaging
Imports MyFriends.Helpers
Imports MyFriends.Model
Imports Windows.UI.Xaml.Media.Imaging

Namespace ViewModel
    Public Class FriendViewModel
        Inherits ViewModelBase

        Private _schema As FullNameSchema = FullNameSchema.FirstLast

        Private _model As [Friend]
        Public Property Model As [Friend]
            Get
                Return _model
            End Get
            Private Set(ByVal value As [Friend])
                _model = value
            End Set
        End Property

        Public Sub New(ByVal model As [Friend], ByVal schema As FullNameSchema)
            Me.Model = model

            AddHandler Me.Model.PropertyChanged, Sub(s, e)
                                                     If e.PropertyName = [Friend].FirstNamePropertyName OrElse e.PropertyName = [Friend].LastNamePropertyName Then
                                                         RaisePropertyChanged(Function() FullName)
                                                         Exit Sub
                                                     End If

                                                     If e.PropertyName = [Friend].DateOfBirthPropertyName Then
                                                         RaisePropertyChanged(Function() DateOfBirthFormatted)
                                                     End If
                                                 End Sub

            Messenger.Default.Register(Of ChangeFullNameSchemaMessage)(Me, Sub(msg)
                                                                               _schema = msg.Schema
                                                                               RaisePropertyChanged(Function() FullName)
                                                                           End Sub)
        End Sub

        Public ReadOnly Property DateOfBirthFormatted As String
            Get
                Return Model.DateOfBirth.ToString("d")
            End Get
        End Property

        Public ReadOnly Property FullName As String
            Get
                Select Case _schema
                    Case FullNameSchema.LastFirstComma
                        Return String.Format("{0}, {1}", Model.LastName, Model.FirstName)
                    Case Else
                        Return String.Format("{0} {1}", Model.FirstName, Model.LastName)
                End Select
            End Get
        End Property
    End Class
End Namespace
