Imports System.Collections.ObjectModel
Imports GalaSoft.MvvmLight
Imports GalaSoft.MvvmLight.Command
Imports GalaSoft.MvvmLight.Messaging
Imports MyFriends.Helpers
Imports MyFriends.Model
Imports PresenterBuddy.Helpers

Namespace ViewModel
    ''' <summary>
    ''' This class contains properties that the main View can data bind to.
    ''' <para>
    ''' See http://www.galasoft.ch/mvvm
    ''' </para>
    ''' </summary>
    Public Class MainViewModel
        Inherits ViewModelBase

        Private ReadOnly _dataService As IDataService
        Private ReadOnly _navigationService As INavigationService

        ''' <summary>
        ''' Initializes a new instance of the MainViewModel class.
        ''' </summary>
        Public Sub New(ByVal dataService As IDataService, ByVal navigationService As INavigationService)
            _dataService = dataService
            _navigationService = navigationService
            Friends = New ObservableCollection(Of FriendViewModel)

#If DEBUG Then
            CreateDesignTimeData()
#End If

        End Sub

        Private _friends As ObservableCollection(Of FriendViewModel)
        Public Property Friends As ObservableCollection(Of FriendViewModel)
            Get
                Return _friends
            End Get
            Private Set(ByVal value As ObservableCollection(Of FriendViewModel))
                _friends = value
            End Set
        End Property

        Private _refreshCommand As RelayCommand
        Public ReadOnly Property RefreshCommand As RelayCommand
            Get
                If _refreshCommand IsNot Nothing Then
                    Return _refreshCommand
                Else
                    _refreshCommand = New RelayCommand(AddressOf ExecuteRefreshCommand)
                    Return _refreshCommand
                End If
            End Get
        End Property

        Private Async Sub ExecuteRefreshCommand()
            Dim friends As IList(Of MyFriends.Model.Friend) = Await _dataService.GetFriends

            If friends IsNot Nothing Then
                Me.Friends.Clear()

                Dim _friend As [Friend]
                For Each _friend In friends
                    Me.Friends.Add(New FriendViewModel(_friend, _schema))
                Next _friend
            End If
        End Sub

        Private _schema As FullNameSchema = FullNameSchema.FirstLast
        Public Property IsNameDisplayFirstLast As Boolean
            Get
                Return _schema = FullNameSchema.FirstLast
            End Get
            Set(ByVal value As Boolean)
                If value Then
                    SetSchema(FullNameSchema.FirstLast)
                End If
            End Set
        End Property

        Private Sub SetSchema(ByVal schema As FullNameSchema)
            _schema = schema
            RaisePropertyChanged(Function() IsNameDisplayFirstLast)
            RaisePropertyChanged(Function() IsNameDisplayLastFirstComma)
            Messenger.Default.Send(New ChangeFullNameSchemaMessage(_schema))
        End Sub

        Public Property IsNameDisplayLastFirstComma As Boolean
            Get
                Return _schema = FullNameSchema.LastFirstComma
            End Get
            Set(ByVal value As Boolean)
                If value Then
                    SetSchema(FullNameSchema.LastFirstComma)
                End If
            End Set
        End Property

        ''' <summary>
        ''' The <see cref="SelectedFriend" /> property's name.
        ''' </summary>
        Public Const SelectedFriendPropertyName As String = "SelectedFriend"

        Private _selectedFriend As FriendViewModel

        ''' <summary>
        ''' Sets and gets the SelectedFriend property.
        ''' Changes to that property's value raise the PropertyChanged event. 
        ''' </summary>
        Public Property SelectedFriend As FriendViewModel
            Get
                Return _selectedFriend
            End Get
            Set(ByVal value As FriendViewModel)
                If MyBase.Set(SelectedFriendPropertyName, _selectedFriend, value) AndAlso value IsNot Nothing Then
                    _navigationService.Navigate(GetType(FriendView))
                End If
            End Set
        End Property

#If DEBUG Then
        Private Sub CreateDesignTimeData()
            If IsInDesignMode Then
                RefreshCommand.Execute(Nothing)
                SelectedFriend = Friends(0)
            End If
        End Sub
#End If

    End Class
End Namespace
